@@ -1,12 +1,13 @@
 /* Prototypes for exported functions defined in mcore.c
-   Copyright (C) 2000, 2002, 2003, 2004 Free Software Foundation, Inc.
+   Copyright (C) 2000, 2002, 2003, 2004, 2005, 2007
+   Free Software Foundation, Inc.
    Contributed by Nick Clifton (nickc@redhat.com)
 
    This file is part of GCC.
 
    GCC is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GCC is distributed in the hope that it will be useful,
@@ -15,20 +16,19 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GCC; see the file COPYING.  If not, write to
-   the Free Software Foundation, 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   along with GCC; see the file COPYING3.  If not see
+   <http://www.gnu.org/licenses/>.  */
 
 extern const char * mcore_output_jump_label_table	(void);
 extern void         mcore_expand_prolog          	(void);
 extern void         mcore_expand_epilog          	(void);
-extern int          mcore_const_ok_for_inline    	(long);
-extern int          mcore_num_ones               	(int);
-extern int          mcore_num_zeros              	(int);
+extern int          mcore_const_ok_for_inline    	(HOST_WIDE_INT);
+extern int          mcore_num_ones               	(HOST_WIDE_INT);
+extern int          mcore_num_zeros              	(HOST_WIDE_INT);
 extern int          mcore_initial_elimination_offset	(int, int);
 extern int          mcore_byte_offset            	(unsigned int);
 extern int          mcore_halfword_offset        	(unsigned int);
-extern int          mcore_const_trick_uses_not   	(long);
+extern int          mcore_const_trick_uses_not   	(HOST_WIDE_INT);
 extern void         mcore_override_options       	(void);
 extern int          mcore_dllexport_name_p       	(const char *);
 extern int          mcore_dllimport_name_p       	(const char *);
@@ -36,31 +36,27 @@ extern int          mcore_naked_function_p       	(void);
 
 #ifdef TREE_CODE
 #ifdef HAVE_MACHINE_MODES
-extern int          mcore_num_arg_regs           	(enum machine_mode, tree);
+extern int          mcore_num_arg_regs           	(enum machine_mode, const_tree);
 #endif /* HAVE_MACHINE_MODES */
 
 #ifdef RTX_CODE
-extern rtx          mcore_function_value         	(tree, tree);
+extern rtx          mcore_function_value         	(const_tree, const_tree);
 #endif /* RTX_CODE */
 #endif /* TREE_CODE */
 
 #ifdef RTX_CODE
 
-extern GTY(()) rtx arch_compare_op0;
-extern GTY(()) rtx arch_compare_op1;
-
 extern const char * mcore_output_bclri         		(rtx, int);
 extern const char * mcore_output_bseti         		(rtx, int);
 extern const char * mcore_output_cmov          		(rtx *, int, const char *);
 extern char *       mcore_output_call          		(rtx *, int);
 extern int          mcore_is_dead                	(rtx, rtx);
 extern int          mcore_expand_insv            	(rtx *);
-extern int          mcore_modify_comparison      	(RTX_CODE);
 extern bool         mcore_expand_block_move      	(rtx *);
 extern const char * mcore_output_andn          		(rtx, rtx *);
 extern void         mcore_print_operand_address  	(FILE *, rtx);
 extern void         mcore_print_operand          	(FILE *, rtx, int);
-extern rtx          mcore_gen_compare_reg        	(RTX_CODE);
+extern bool         mcore_gen_compare	        	(RTX_CODE, rtx, rtx);
 extern int          mcore_symbolic_address_p     	(rtx);
 extern bool         mcore_r15_operand_p			(rtx);
 extern enum reg_class mcore_secondary_reload_class	(enum reg_class, enum machine_mode, rtx);
@@ -71,25 +67,7 @@ extern int          mcore_arith_S_operand         	(rtx);
 #ifdef HAVE_MACHINE_MODES
 extern const char * mcore_output_move          		(rtx, rtx *, enum machine_mode);
 extern const char * mcore_output_movedouble    		(rtx *, enum machine_mode);
-extern int          mcore_arith_reg_operand       	(rtx, enum machine_mode);
-extern int          mcore_general_movsrc_operand  	(rtx, enum machine_mode);
-extern int          mcore_general_movdst_operand  	(rtx, enum machine_mode);
-extern int          mcore_reload_operand          	(rtx, enum machine_mode);
-extern int          mcore_arith_J_operand         	(rtx, enum machine_mode);
-extern int          mcore_arith_K_operand         	(rtx, enum machine_mode);
-extern int          mcore_arith_K_operand_not_0   	(rtx, enum machine_mode);
-extern int          mcore_arith_M_operand         	(rtx, enum machine_mode);
-extern int          mcore_arith_K_S_operand       	(rtx, enum machine_mode);
-extern int          mcore_arith_imm_operand       	(rtx, enum machine_mode);
-extern int          mcore_arith_any_imm_operand   	(rtx, enum machine_mode);
-extern int          mcore_arith_O_operand         	(rtx, enum machine_mode);
-extern int          mcore_literal_K_operand       	(rtx, enum machine_mode);
-extern int          mcore_addsub_operand          	(rtx, enum machine_mode);
-extern int          mcore_compare_operand         	(rtx, enum machine_mode);
-extern int          mcore_load_multiple_operation 	(rtx, enum machine_mode);
-extern int          mcore_store_multiple_operation	(rtx, enum machine_mode);
-extern int          mcore_call_address_operand    	(rtx, enum machine_mode);
-
+extern int          const_ok_for_mcore                  (HOST_WIDE_INT);
 #ifdef TREE_CODE
 extern rtx          mcore_function_arg           	(CUMULATIVE_ARGS, enum machine_mode, tree, int);
 #endif /* TREE_CODE */
